@@ -13,8 +13,8 @@ if qt_bindings == 'PyQt4':
     sip.setapi('QString', 2)
     sip.setapi('QVariant', 2)
 
-from dockers import *
-from calendar import CalendarView
+from .dockers import *
+from .calendar import CalendarView, CalendarTopNav
 
-__version_info__ = ['0', '2', '1']
+__version_info__ = ['0', '3', '0']
 __version__ = '.'.join(__version_info__)
