@@ -7,9 +7,11 @@
 ##############################################################################
 
 from qtalchemy import PBTableModel, ModelColumn, Signal, Slot
+from qtalchemy import LayoutLayout, LayoutWidget
 from qtalchemy.widgets import TableView
 from PySide import QtCore, QtGui
 from .utils import *
+import fuzzyparsers
 import datetime
 
 day_names = "Sunday Monday Tuesday Wednesday Thursday Friday Saturday".split(' ')
@@ -179,7 +181,7 @@ class CalendarView(TableView):
                 this_day_list = [e for e in events if e.start_date <= d and e.end_date >= d]
                 calWeek.append(this_day_list)
 
-                zz = range(len(this_day_list))
+                zz = list(range(len(this_day_list)))
                 for e in this_day_list:
                     if e.visual_row_level in zz:
                         zz.remove(e.visual_row_level)
@@ -210,25 +212,67 @@ class CalendarView(TableView):
         index = QtCore.QModelIndex() # TODO:  write this code
         m.select(index, selMode)
 
-    def mouseDoubleClickEvent(self, event):
-        index = self.indexAt(event.pos())
+    def itemAt(self, pos):
+        index = self.indexAt(pos)
         if index is not None and index.isValid():
             for entry in index.internalPointer().entryList(index):
                 eventRect = index.internalPointer().entryBlock(entry, index, self.visualRect(index))
-                if eventRect.contains(event.pos()):
-                    self.doubleClickCalendarEvent.emit(entry.obj)
-                    event.accept()
-                    break
+                if eventRect.contains(pos):
+                    return entry.obj
+        return None
+
+    def mouseDoubleClickEvent(self, event):
+        obj = self.itemAt(event.pos())
+        if obj is not None:
+            self.doubleClickCalendarEvent.emit(obj)
+            event.accept()
         super(CalendarView, self).mouseDoubleClickEvent(event)
 
     def contextMenuEvent(self, event):
-        index = self.indexAt(event.pos())
-        if index is not None and index.isValid():
-            for entry in index.internalPointer().entryList(index):
-                eventRect = index.internalPointer().entryBlock(entry, index, self.visualRect(index))
-                if eventRect.contains(event.pos()):
-                    self.contextMenuCalendarEvent.emit(event.pos(), entry.obj)
-                    event.accept()
-                    break
+        obj = self.itemAt(event.pos())
+        if obj is not None:
+            self.contextMenuCalendarEvent.emit(event.pos(), obj)
+            event.accept()
         super(CalendarView, self).contextMenuEvent(event)
 
+class CalendarTopNav(QtGui.QWidget):
+    """
+    >>> app = qtapp()
+    >>> c = CalendarTopNav()
+    """
+    relativeMove = Signal(int)
+    absoluteMove = Signal(object)
+
+    def __init__(self, parent=None):
+        super(CalendarTopNav, self).__init__(parent)
+
+        main = QtGui.QHBoxLayout(self)
+        main.setContentsMargins(0, 0, 0, 0)
+        self.earlier = [
+            LayoutWidget(main, QtGui.QPushButton("<<<")),
+            LayoutWidget(main, QtGui.QPushButton("<<")),
+            LayoutWidget(main, QtGui.QPushButton("<"))]
+        self.earlier.reverse()
+        self.month_label = LayoutWidget(main, QtGui.QLabel("&Month:"))
+        self.month = LayoutWidget(main, QtGui.QLineEdit())
+        self.later = [
+            LayoutWidget(main, QtGui.QPushButton(">")),
+            LayoutWidget(main, QtGui.QPushButton(">>")),
+            LayoutWidget(main, QtGui.QPushButton(">>>"))]
+        for b in self.earlier + self.later:
+            b.setMaximumWidth(40)
+        self.month_label.setBuddy(self.month)
+        self.month.returnPressed.connect(self.input_reset)
+        self.setFocusProxy(self.month)
+        self.setMaximumHeight(self.month.sizeHint().height())
+
+        for i in range(3):
+            self.earlier[i].clicked.connect(lambda index=-i-1:self.relativeMove.emit(index))
+            self.later[i].clicked.connect(lambda index=+i+1:self.relativeMove.emit(index))
+
+    def input_reset(self):
+        try:
+            x = fuzzyparsers.parse_date(self.month.text())
+            self.absoluteMove.emit(x)
+        except Exception as e:
+            pass
